package com.onresolve.jira.groovy.canned.utils

import com.atlassian.jira.issue.search.SearchRequest
import com.atlassian.jira.issue.search.SearchResults
import com.atlassian.jira.issue.search.SearchProvider
import com.atlassian.jira.web.bean.PagerFilter
import com.atlassian.jira.ComponentManager
import com.atlassian.jira.security.JiraAuthenticationContext
import org.apache.log4j.Category
import com.atlassian.jira.bc.filter.SearchRequestService
import com.atlassian.jira.bc.JiraServiceContextImpl
import com.atlassian.jira.bc.JiraServiceContext
import com.atlassian.jira.jql.builder.JqlQueryBuilder
import com.atlassian.query.Query

class FilterUtils {

    public static Category log = Category.getInstance(FilterUtils.class)
    static ComponentManager componentManager = ComponentManager.getInstance()
    static SearchProvider searchProvider = componentManager.getSearchProvider()
    static JiraAuthenticationContext authenticationContext = componentManager.getJiraAuthenticationContext()
    static SearchRequestService searchRequestService = componentManager.getSearchRequestService()
    static JiraServiceContext ctx = new JiraServiceContextImpl(authenticationContext.getUser());

    public static String FIELD_FILTER_ID = "FIELD_FILTER_ID"
    public static String FIELD_PROJECT_ID = "FIELD_PROJECT_ID"

    public static SearchResults getSearchResults(SearchRequest sr) {
        SearchResults results

        results = searchProvider.search(sr.getQuery(), authenticationContext.getUser(), PagerFilter.getUnlimitedFilter())
        results
    }

    public static SearchRequest getOrCreateSearchRequest(Map params) {
        String filterId = params[FIELD_FILTER_ID] as String
        String projectId = params[FIELD_PROJECT_ID]
        SearchRequest sr

        if (filterId) {
            sr = searchRequestService.getFilter(ctx, filterId as Long)
        }
        else {
            JqlQueryBuilder jqlQueryBuilder = JqlQueryBuilder.newBuilder()
            Query query = jqlQueryBuilder.where().project(projectId).buildQuery()
            sr = new SearchRequest(query)
            log.debug(sr)
        }
        sr
    }
}
